% This code reads a GrADS file and converts it to a netCDF file.
% Original programmer: Damien Decremer (damien.decremer@gmail.com)


clc; clear; close all;
% ------ CHANGE HERE AND NOWHERE ELSE ---------
file = 'Totalaer3_allsky_3DQ';       % Data name without .ctl. e.g. if dataset is MODIS-CLIM-T42.ctl, then file = 'MODIS-CLIM-T42'

% % sometimes, Z-levels are used for something else than pressure or height. Specify the unit of Z-levels here
% zvarUnits = 'nm'; % e.g. 'mbar' or 'hPa' (default)        
% zvarLong_name = 'wavelengths'; % name of Z-levels unit, e.g. 'altitude', 'pressure_level' or 'height' (default)
% zvarDescending = false; % e.g. for pressure levels z1=1000hPa and zn=1hPa, thus z1 > zn => zvarDescending = true (default)
% % GrADS accepts the following units of height
% % common units for pressure: 'bar', 'mbar', 'dbar', 'atm', 'Pa' or 'hPa' (default)
% % common units for length: 'm', 'cm', 'dm', 'ft' or 'km'
% % common dimensionless units: 'level', 'layer' or 'sigma_level'
% % sometimes units of 'density' or 'temperature' are used for height as well

interactiveComments = 0;       % Adds comments from the ctl file into the netcdf file automatically when =0, otherwise (=1) asks confirmation for each comment line interactively

% FOR AEROCOM ONLY
institution = 'DRI';
contact = 'Eddy.Chung@dri.edu';
source = '';
project_id = '';
experiment_id = '';
% -----------------------------

% Import all variables from the ctl file
gradsRead([file,'.ctl']);
% load all.mat;

% Create the file
[ncid, status] = netcdf.create([file,'.nc'], 'NC_SHARE');
% [ncid, status] = netcdf.create([file,'.nc'], 'NETCDF4');
% Define the dimensions
x_dimid = netcdf.defDim(ncid,'lon',header.XDEF.num);
y_dimid = netcdf.defDim(ncid,'lat',header.YDEF.num);
% Finding all duplicate z dimensions to define in netCDF file later
levs = zeros(1,size(header.VARS,1));
for i = 1:size(header.VARS,1)
    levs(i) = header.VARS(i,1).levs;
end
dimCounter = 0;
z_dimid = zeros(1,size(levs,2)); z_dimid(:) = NaN;
for i = 1:size(levs,2)
    if (levs(i) > 1)
        currentDims = find(levs == levs(i));
        if (length(currentDims) <= 1 || i == min(currentDims))
            dimCounter = dimCounter + 1;
            z_dimid(i) = eval(['netcdf.defDim(ncid,''z',num2str(dimCounter),''',levs(',num2str(i),'));']);
        else
            z_dimid(i) = z_dimid(currentDims(1));
        end
    end
end
% In case there is only one z-level, define only one
if (dimCounter == 1)
    netcdf.abort(ncid);
    [ncid, status] = netcdf.create([file,'.nc'], 'NC_SHARE');
%     [ncid, status] = netcdf.create([file,'.nc'], 'NETCDF4');
%     ncid = netcdf.open([file,'.nc'],'NC_WRITE');
%     netcdf.reDef(ncid);
    x_dimid = netcdf.defDim(ncid,'lon',header.XDEF.num);
    y_dimid = netcdf.defDim(ncid,'lat',header.YDEF.num);
    z_dimid = netcdf.defDim(ncid,'lev',header.ZDEF.num);
end
t_dimid = netcdf.defDim(ncid,'time',header.TDEF.num);
% e_dimid = netcdf.defDim(ncid,'e',1);

% Define the coordinate variables
x_varid = netcdf.defVar(ncid,'lon','float',x_dimid);
y_varid = netcdf.defVar(ncid,'lat','float',y_dimid);
% The varids MUST be exactly the same as the dimids
dimCounter = 0;
for i = 1:size(z_dimid,2)
    if (~isnan(z_dimid(i)))
        dimCounter = dimCounter + 1;
        if (size(z_dimid,2) <= 1)
            z_varid =  netcdf.defVar(ncid,'lev','float',z_dimid);
        else
            z_varid(i) =  eval(['netcdf.defVar(ncid,''lev',num2str(dimCounter),''',''float'',z_dimid(',num2str(i),'));']);
        end
    else
        z_varid(i) = z_dimid(i);
    end
end
t_varid = netcdf.defVar(ncid,'time','float',t_dimid);
% e_varid = netcdf.defVar(ncid,'e','float',e_dimid);


% Assign units attributes to coordinate variables.
netcdf.putAtt(ncid, x_varid,'units','degrees_east');
netcdf.putAtt(ncid, x_varid,'long_name','longitude');
netcdf.putAtt(ncid, x_varid,'standard_name','longitude');
netcdf.putAtt(ncid, x_varid,'axis','X'); 
netcdf.putAtt(ncid, y_varid,'units','degrees_north');
netcdf.putAtt(ncid, y_varid,'long_name','latitude');
netcdf.putAtt(ncid, y_varid,'standard_name','latitude');
netcdf.putAtt(ncid, y_varid,'axis','Y'); 

if (~exist('zvarUnits'))
    zvarUnits = 'hPa';
    zvarLong_name = 'height';
    zvarDescending = true;
end
for i = 1:length(z_varid)
    if (~isnan(z_varid(i)))
        netcdf.putAtt(ncid, z_varid(i),'units',zvarUnits);
        netcdf.putAtt(ncid, z_varid(i),'long_name',zvarLong_name);
        netcdf.putAtt(ncid, z_varid(i),'standard_name',zvarLong_name);
        netcdf.putAtt(ncid, z_varid(i),'axis','Z'); 
        if (zvarDescending)
            netcdf.putAtt(ncid, z_varid(i),'positive','down'); % default, e.g. pressure levels
        else
            netcdf.putAtt(ncid, z_varid(i),'positive','up'); 
        end
    end
end
netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),'TITLE',header.TITLE);
% The following code adds any comments found in the code as global
% attributes on user request.
ctl_id = fopen(deblank([file,'.ctl']),'r');
infoCounter = 0;
while ~feof(ctl_id)
    line=fgets(ctl_id);
    if isempty(line(1:end-1)), break, end
    % cutting the line into words.
    line_gaps=isspace(line);
    % removing consecutive blancks.
    while line_gaps(1)==1, line_gaps(1)=[]; line(1)=[]; end
    if (line(1)=='*' && ~isempty(line(2:end-1)))
        disp(['Found the following comment line: ',strtrim(line(2:end-1))]);
        if (interactiveComments)
            reply = input('Do you want to add this line to the netCDF file? y/n [y]: ', 's');
        else
            reply = 'y';
        end
        disp(' ');
        if (isempty(reply))
            reply = 'y';
        end
        if (strcmp(reply,'Y') || strcmp(reply,'y'))
            infoCounter = infoCounter + 1;
            netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),['Info',num2str(infoCounter)],strtrim(line(2:end)));
        end
    end
end
if (exist('institution')); netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),'institution',institution); end
if (exist('contact')); netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),'contact',contact); end
if (exist('source')); netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),'source',source); end
if (exist('project_id')); netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),'project_id',project_id); end
if (exist('experiment_id')); netcdf.putAtt(ncid, netcdf.getConstant('NC_GLOBAL'),'experiment_id',experiment_id); end



% The following lines are used to build the timeString, which is the
% attribute for the time dimension.
first = datevec(header.TDEF.vec(1));
firstYear = first(1);
firstMonth = first(2);
firstDay = first(3);
firstTime = sprintf('%02i:%02i',first(4),first(5));
months = ['Jan';'Feb';'Mar';'Apr';'May';'Jun';'Jul';'Aug';'Sep';'Oct';'Nov';'Dec'];
firstMonth = months(firstMonth,:);
incStrings = {'years';'months';'days';'hours';'minutes'};
incNums = round(10000*[header.TDEF.vec(2)/365.25,...
    header.TDEF.vec(2)/30.4375,...
    header.TDEF.vec(2),...
    header.TDEF.vec(2)*24,...
    header.TDEF.vec(2)*1440])/10000;
roundedIncNums = fix(incNums);
incNums(find(roundedIncNums == 0)) = Inf;
incNums(find(incNums ~= roundedIncNums)) = Inf;
incNums(find(incNums > min(incNums))) = 0;
% timeString = [incStrings(find(incNums~=0),:),' since ',num2str(firstMonth),' ',num2str(firstDay),', ',num2str(firstYear),', ',firstTime,' UTC'];
% netcdf.putAtt(ncid, t_varid,'long_name',timeString);
% netcdf.putAtt(ncid, t_varid,'standard_name',timeString);
timeString = [incStrings{find(incNums~=0),:},sprintf(' since %04i-%02i-%02i %02i:%02i',first(1),first(2),first(3),first(4),first(5))];
netcdf.putAtt(ncid, t_varid,'units',timeString);
netcdf.putAtt(ncid, t_varid,'long_name','time');
netcdf.putAtt(ncid, t_varid,'standard_name','time');
netcdf.putAtt(ncid, t_varid,'axis','T');


% netcdf.putAtt(ncid, e_varid,'grads_dim','e');
% netcdf.putAtt(ncid, e_varid,'axis','E');

for i = 1:size(header.VARS,1)
    % The dimids array is used to pass the dimids of the dimensions of
    % the netCDF variables. Both of the netCDF variables we are
    % creating share the same four dimensions. In both C and MATLAB, the
    % unlimited dimension must come first on the list of dimids.
    if(levs(i) == 1)
        dimids = [x_dimid, y_dimid, t_dimid];
    else
        if (length(z_dimid) == 1)
            dimids = [x_dimid, y_dimid, z_dimid, t_dimid];
        else
            dimids = [x_dimid, y_dimid, z_dimid(i), t_dimid];
        end
    end
    % Define the netCDF variables
    eval(['var',num2str(i),'_id = netcdf.defVar(ncid,header.VARS(i,1).name,''double'',dimids);']);
    % Assign units attributes to the netCDF variables.
    if (isempty(strfind(header.VARS(i,1).descr,'(')))
        warning('One of the variables doesn''t have a unit specified.');
    else
        eval(['netcdf.putAtt(ncid, var',num2str(i),'_id, ''units'',header.VARS(',num2str(i),',1).descr(strfind(header.VARS(',num2str(i),',1).descr,''(''):1:strfind(header.VARS(',num2str(i),',1).descr,'')'')));']);
    end
    eval(['netcdf.putAtt(ncid, var',num2str(i),'_id, ''standard_name'',header.VARS(',num2str(i),',1).descr);']);
%     eval(['if (sum(sum(sum(sum(sum(sum(sum(sum(sum(sum(isnan(',header.VARS(i,1).name,'))))))))))) > 0); netcdf.defVarFill(ncid, var',num2str(i),'_id,false,',num2str(header.UNDEF),'); end']);
end

% Leave define mode and enter data mode to write data.
netcdf.endDef(ncid);

% Write the coordinate variable data. This will put the latitudes
% and longitudes of our data grid into the netCDF file. 
netcdf.putVar(ncid,x_dimid,single(header.XDEF.vec));
netcdf.putVar(ncid,y_dimid,single(header.YDEF.vec));
for i = 1:size(z_dimid,2)
    if (size(z_dimid,2) > 1)
        if (~isnan(z_dimid(i)))
            netcdf.putVar(ncid,z_dimid(i),single(header.ZDEF.vec(1:levs(i))));
        end
    else
        if (~isnan(z_dimid(i)))
            netcdf.putVar(ncid,z_dimid,single(header.ZDEF.vec));
        end
    end
end
netcdf.putVar(ncid,t_dimid,[1:header.TDEF.num]);

for i = 1:size(header.VARS,1)
    eval(['netcdf.putVar(ncid,var',num2str(i),'_id,single(',header.VARS(i,1).name,'));']);
end

% Close the file. 
netcdf.close(ncid);
clear;
% test the netcdf file
% clear; close all; clc;
% file = 'Indian1';
% [vars varatts dims] = ncdfRead([file,'.nc']);
